function mri_siso()
% mri (Appendix fMRI & fNIRS Dynamic Causal Model SISO)
% by Christian Himpe, 2016 ( wwwmath.uni-muenster.de/u/himpe )
% released under BSD 2-Clause License ( opensource.org/licenses/BSD-2-Clause )
%*
    addpath('../');

%% SETUP

    J = 1;
    N = 4;
    O = 1;
    T = [0.1,20.0];
    L = floor(T(2)/T(1)) + 1;

    XN = [0;1;1;1];
    U = [ones(J,1)./T(1),zeros(J,L)];

    kappa = 0.65;
    gamma = 0.41;
    tau = 0.98;
    alpha = 0.32;
    rho = 0.34;

    rtau = 1.0/tau;
    ralpha = 1.0/alpha;

    V0 = 0.02;
    k1 = 7.0*rho;
    k2 = 2.0*rho - 0.2;

    NON = @(x,u,p) [ u - kappa * x(1) - gamma * ( x(2) - 1.0 ); ...
                     x(1); ...
                     rtau * ( x(2) - x(3).^ralpha ); ...
                     rtau * ( x(2) .* (1.0 - (1.0 - rho).^(1.0 ./ x(2) ) ) ./ rho  - ( x(4) .* x(3).^(ralpha - 1.0) ) )];

    NOU = @(x,u,p) V0 * ( k1 * (1.0 - x(4)) + k2 * (1.0 - x(3)) );

    o = tic;
    WX = emgr(NON,NOU,[J,N,O],T,'x',1,[1,0,0,3,0,0,0,0,0,0,0,0],1.0,0,XN,1.0,0.9);
    [UX,DX,vx] = svd(WX);
    OFFLINE_WJN = toc(o)
    MRI_HSV = diag(DX)

end
