function ERR = cerror(ODE,DYN,OUT,T,X,U,P,UX,VX,UP,n)
% cerror (combined state and parameter error evaluation)
% by Christian Himpe, 2014-2016 ( wwwmath.uni-muenster.de/u/himpe )
% released under BSD 2-Clause License ( opensource.org/licenses/BSD-2-Clause )
%*
    N = size(UX,2);
    M = size(P,1);
    Q = size(P,2);

    TMP = zeros(floor(N/n)-1,floor(M/n)-1,3,Q);

    for I=1:Q

        Y = ODE(DYN,OUT,T,X,U,P(:,I));

        n1 = norm(Y(:),1);
        n2 = norm(Y(:),2);
        n8 = norm(Y(:),Inf);

        j = 1;

        for J=1:n:(M-n)

            up = UP(:,1:J);
            vp = up';

            p = vp*P(:,I);

            k = 1;

            for K=1:n:(N-n)

                ux = UX(:,1:K);
                vx = VX(1:K,:);

                if(any(X))
                    dyn = @(x,u,p) vx*DYN(X + ux*x,u,p);
                    out = @(x,u,p) OUT(X + ux*x,u,p);
                else
                    dyn = @(x,u,p) vx*DYN(ux*x,u,p);
                    out = @(x,u,p) OUT(ux*x,u,p);
                end

                x = zeros(K,1);

                y = ODE(dyn,out,T,x,U,up*p);

                TMP(k,j,1,I) = norm(Y(:)-y(:),1)/n1;
                TMP(k,j,2,I) = norm(Y(:)-y(:),2)/n2;
                TMP(k,j,3,I) = norm(Y(:)-y(:),Inf)/n8;

                k = k + 1;
            end

            j = j + 1;
        end
    end

    ERR = sqrt(sum(TMP.^2,4));
    ERR(:,:,4) = max(TMP(:,:,2,:),[],4);
end
