function ch5plot(o)
% ch5plot (plot results of chapter 5)
% by Christian Himpe, 2014-2015 ( http://wwwmath.uni-muenster.de/u/himpe )
% released under BSD 2-Clause License ( opensource.org/licenses/BSD-2-Clause )
%*
    addpath('../');

    if(nargin<1 || isempty(o)), o = 0; end;


%% SETUP

    l1e = 'l_1 Error';
    l2e = 'l_2 Error';
    l8e = 'l_\infty Error';

    l1b = 'l_1 Bound';
    l2b = 'l_2 Bound';
    l8i = 'l_\infty Indic';

    l1xl2e = 'l_1 \otimes l_2 Error';
    l2xl2e = 'l_2 \otimes l_2 Error';
    l8xl2e = 'l_\infty \otimes l_2 Error';

    l2xl8a = 'l_2 \otimes l_\infty Max';
    l2xl8b = 'l_2 \otimes l_\infty Avg';
    l2xl8c = 'l_2 \otimes l_\infty Min';

    lne = 'northeast';
    lsw = 'southwest';

    pf = ['../../plots/',mfilename(),'_'];

    linewidth = 8;

%% EMGR DATA

    load('../../data/ch5emgr.mat');

    figure;
    loglog([0.01,0.005,0.001,0.0005,0.0001],ERR01(3,:)','-*','Color',colors('red'),'linewidth',linewidth);
    hold on;
    loglog([0.01,0.005,0.001,0.0005,0.0001],ERR01(4,:)','-*','Color',colors('green'),'linewidth',linewidth);
    loglog([0.01,0.005,0.001,0.0005,0.0001],ERR01(5,:)','-*','Color',colors('blue'),'linewidth',linewidth);
    hold off;
    xlim([0.0001,0.01]);
    set(gca,'XDir','reverse');
    legend('|WC-WO|','|WO-WX|','|WC-WX|');
    set([gca; findall(gca,'Type','text')],'FontSize',14);
    pbaspect([1.5,1.6,1]);
    print('-dpdf',['../../plots/ch5emgr_ver0a.pdf']);

    figure;
    semilogy([1,2,4,8,10],ERR02(1,:),'-*','Color',colors('red'),'linewidth',linewidth);
    hold on;
    semilogy([1,2,4,8,10],ERR02(2,:),'-*','Color',colors('green'),'linewidth',linewidth);
    semilogy([1,2,4,8,10],ERR02(3,:),'-*','Color',colors('blue'),'linewidth',linewidth);
    hold off;
    xlim([1,10]);
    legend('|WC-W|','|WO-W|','|WX-W|');
    set([gca; findall(gca,'Type','text')],'FontSize',14);
    pbaspect([1.5,1.6,1]);
    print('-dpdf',['../../plots/ch5emgr_ver0b.pdf']);

    minerr = 1e-16;

    plot_error([ERR11;real(BND11)],'ch5emgr_WCWO',N,minerr,{l1e,l2e,l8e,l1b,l2b,l8i},lne,0);	% Balanced Truncation
    plot_error([ERR12;real(BND12)],'ch5emgr_WCWo',N,minerr,{l1e,l2e,l8e,l1b,l2b,l8i},lne,0);	% Linear Balanced Truncation
    plot_error([ERR13;real(BND13)],'ch5emgr_WY',N,minerr,{l1e,l2e,l8e,l1b,l2b,l8i},lne,0);	% Linear Cross Gramian
    plot_error([ERR14;real(BND14)],'ch5emgr_WX',N,minerr,{l1e,l2e,l8e,l1b,l2b,l8i},lne,0);	% Cross Gramian
    plot_error([ERR15;real(BND15)],'ch5emgr_Wx',N,minerr,{l1e,l2e,l8e,l1b,l2b,l8i},lne,0);	% Balanced Gains Cross Gramian
    plot_error(ERR16,'ch5emgr_WZ',N,minerr,{l1e,l2e,l8e},lne,0);				% Non-Symmetric Cross Gramian

    minerr = 1e-14;

    plot_error(ERR21,'ch5emgr_pWCWO',N,minerr,{l1e,l2e,l8e},lne,0);		% Balanced Truncation
    plot_error(ERR22,'ch5emgr_pWCWo',N,minerr,{l1e,l2e,l8e},lne,0);		% Linear Balanced Truncation
    plot_error(ERR23,'ch5emgr_pWY',N,minerr,{l1e,l2e,l8e},lne,0);		% Linear Cross Gramian
    plot_error(ERR24,'ch5emgr_pWX',N,minerr,{l1e,l2e,l8e},lne,0);		% Cross Gramian
    plot_error(ERR25,'ch5emgr_pWx',N,minerr,{l1e,l2e,l8e},lne,0);		% Balanced Gains Cross Gramian
    plot_error(ERR26,'ch5emgr_pWZ',N,minerr,{l1e,l2e,l8e},lsw,0);		% Non-Symmetric Cross Gramian

    plot_error(ERR31,'ch5emgr_WS',P,minerr,{l1xl2e,l2xl2e,l8xl2e},lsw,0);	% Sensitivity Gramian
    plot_error(ERR32,'ch5emgr_Ws',P,minerr,{l1xl2e,l2xl2e,l8xl2e},lsw,0);	% Centered Sensitivity Gramian
    plot_error(ERR33,'ch5emgr_WI',P,minerr,{l1xl2e,l2xl2e,l8xl2e},lne,0);	% Identifiability Gramian
    plot_error(ERR34,'ch5emgr_Wi',P,minerr,{l1xl2e,l2xl2e,l8xl2e},lne,0);	% Approximate Identifiability Gramian (WP)
    plot_error(ERR35,'ch5emgr_WJ',P,minerr,{l1xl2e,l2xl2e,l8xl2e},lne,0);	% Cross-Identifiability Gramian (sym+pinv)
    plot_error(ERR36,'ch5emgr_Wj',P,minerr,{l1xl2e,l2xl2e,l8xl2e},lne,0);	% Approximate Cross-Identifiability Gramian (sym+ainv)

    err41 = [diag(ERR41(:,:,1)),diag(ERR41(:,:,2)),diag(ERR41(:,:,3))]';
    err42 = [diag(ERR42(:,:,1)),diag(ERR42(:,:,2)),diag(ERR42(:,:,3))]';
    err43 = [diag(ERR43(:,:,1)),diag(ERR43(:,:,2)),diag(ERR43(:,:,3))]';

    plot_error(err41,'ch5emgr_wswo',N,minerr,{l1xl2e,l2xl2e,l8xl2e},lsw,0);	% Controllability-Based Combined Reduction (diag)
    plot_error(err42,'ch5emgr_wcwi',N,minerr,{l1xl2e,l2xl2e,l8xl2e},lne,0);	% Observability-Based Combined Reduction (diag)
    plot_error(err43,'ch5emgr_wjwj',N,minerr,{l1xl2e,l2xl2e,l8xl2e},lne,0);	% Observability-Based Combined Reduction (diag)

    plot_surf(ERR41(:,:,2),'ch5emgr_WSWO',[N,P],minerr,0);			% Controllability-Based Combined Reduction
    plot_surf(ERR42(:,:,2),'ch5emgr_WCWI',[N,P],minerr,0);			% Observability-Based Combined Reduction
    plot_surf(ERR43(:,:,2),'ch5emgr_WJWJ',[N,P],minerr,0);			% Cross-Gramian-Based Combined Reduction				

%% OPTMOR DATA

    load('../../data/ch5optmor.mat');

    minerr = 1e-10;

    err01 = [ERR01(4,:,1);ERR01(4,:,2);ERR01(4,:,3)];
    err02 = [ERR02(4,:,1);ERR02(4,:,2);ERR02(4,:,3)];
    err03 = [diag(ERR03(:,:,4,1)),diag(ERR03(:,:,4,2)),diag(ERR03(:,:,4,3))]';

    plot_error(err01,'ch5optmor_ver1',N,minerr,{l2xl8a,l2xl8b,l2xl8c},lne,0);    
    plot_error(err02,'ch5optmor_ver2',N,minerr,{l2xl8a,l2xl8b,l2xl8c},lne,0);
    plot_error(err03,'ch5optmor_ver3',N,minerr,{l2xl8a,l2xl8b,l2xl8c},lne,0); 

    plot_surf(ERR11(:,:,4),'ch5optmor_tk0',[N,P],minerr,0);
    plot_surf(ERR12(:,:,4),'ch5optmor_tk1',[N,P],minerr,0);
    plot_surf(ERR13(:,:,4),'ch5optmor_tk2',[N,P],minerr,0);
    plot_surf(ERR14(:,:,4),'ch5optmor_tk3',[N,P],minerr,0);
    plot_surf(ERR15(:,:,4),'ch5optmor_tk4',[N,P],minerr,0);
    plot_surf(ERR16(:,:,4),'ch5optmor_tk5',[N,P],minerr,0);

    plot_surf(ERR21(:,:,4),'ch5optmor_it0',[N,P],minerr,0);
    plot_surf(ERR22(:,:,4),'ch5optmor_it1',[N,P],minerr,0);
    plot_surf(ERR23(:,:,4),'ch5optmor_it2',[N,P],minerr,0);
    plot_surf(ERR24(:,:,4),'ch5optmor_it3',[N,P],minerr,0);
    plot_surf(ERR25(:,:,4),'ch5optmor_it4',[N,P],minerr,0);
    plot_surf(ERR26(:,:,4),'ch5optmor_it5',[N,P],minerr,0);

    plot_surf(ERR31(:,:,4),'ch5optmor_tr0',[N,P],minerr,0);
    plot_surf(ERR32(:,:,4),'ch5optmor_tr1',[N,P],minerr,0);
    plot_surf(ERR33(:,:,4),'ch5optmor_tr2',[N,P],minerr,0);
    plot_surf(ERR34(:,:,4),'ch5optmor_tr3',[N,P],minerr,0);
    plot_surf(ERR35(:,:,4),'ch5optmor_tr4',[N,P],minerr,0);
    plot_surf(ERR36(:,:,4),'ch5optmor_tr5',[N,P],minerr,0);

    plot_surf(ERR41(:,:,4),'ch5optmor_dd0',[N,P],minerr,0);
    plot_surf(ERR42(:,:,4),'ch5optmor_dd1',[N,P],minerr,0);
    plot_surf(ERR43(:,:,4),'ch5optmor_dd2',[N,P],minerr,0);
    plot_surf(ERR44(:,:,4),'ch5optmor_dd3',[N,P],minerr,0);
    plot_surf(ERR45(:,:,4),'ch5optmor_dd4',[N,P],minerr,0);
    plot_surf(ERR46(:,:,4),'ch5optmor_dd5',[N,P],minerr,0);

%% NONLINEAR DATA

    load('../../data/ch5cmp.mat');

    minerr = 1e-16;

    plot_surf(ERR01(:,:,2),'ch5nrc_wj22',[N,P],minerr,1);
    plot_surf(ERR01(:,:,4),'ch5nrc_wj28',[N,P],minerr,1);

    plot_surf(ERR02(:,:,2),'ch5nrc_opt22',[N,P],minerr,1);
    plot_surf(ERR02(:,:,4),'ch5nrc_opt28',[N,P],minerr,1);

    err11 = [diag(ERR01(:,:,2)),diag(ERR02(:,:,2))]';
    err12 = [diag(ERR01(:,:,4)),diag(ERR02(:,:,4))]';

    plot_error(err11,'ch5nrc_vs22',N,minerr,{'Gramian-Based','Optimization-Based'},lne,1);
    plot_error(err12,'ch5nrc_vs28',N,minerr,{'Gramian-Based','Optimization-Based'},lne,1);

    figure();
    h = barh([[TIM01;zeros(99,1)],TIM02]','stacked');
    h = get(h,'children');
    for I=1:max(numel(TIM02)), set(h{I},'CData',[colors('red');colors('green')]); end
    set(gca,'ytick',[],'YDir','reverse');
    set([gca; findall(gca,'Type','text')],'FontSize',20);
    xlabel('[s]');
    ylabel('Optimization      Gramian  ','rotation',90);
    pbaspect([1,0.6,1]);
    print('-dpdf',['../../plots/ch5nrc_off.pdf']);

    figure;
    semilogy(max(DX,1e-16),'Color',colors('blue'),'linewidth',linewidth);
    set([gca; findall(gca,'Type','text')],'FontSize',18);
    ylim([1e-17,1]);
    pbaspect([1.2,1.4,1]);
    print('../../plots/ch5nrc_wxs.pdf','-dpdf');

    figure;
    semilogy(max(DI,1e-16),'Color',colors('blue'),'linewidth',linewidth);
    set([gca; findall(gca,'Type','text')],'FontSize',18);
    pbaspect([1.2,1.4,1]);
    ylim([1e-17,1]);
    print('../../plots/ch5nrc_wxi.pdf','-dpdf');

    plot_sample(SAM05,'ch5nrc_pns',1);
    plot_sample(SAM06,'ch5nrc_pps',1);
    plot_sample(SAM01,'ch5nrc_g20',1);
    plot_sample(SAM03,'ch5nrc_g50',1);
    plot_sample(SAM02,'ch5nrc_o20',1);
    plot_sample(SAM04,'ch5nrc_o50',1);  
end

