function compare_rc()
% compare_rc (chapter 5 comparison: nonlinear RC ladder)
% by Christian Himpe, 2015-2016 ( wwwmath.uni-muenster.de/u/himpe )
% released under BSD 2-Clause License ( opensource.org/licenses/BSD-2-Clause )
%*
    global TIMINGS;

    addpath('../')

    rand('seed',1009');
    randn('seed',1009');

%% SETUP

    J = 1;
    N = 100;
    O = J;
    T = [0.01,1.0];
    L = floor(T(2)/T(1)) + 1;

    X = zeros(N,1);
    l = 30;
    U = [ones(J,l),zeros(J,L-l)];
    P = N;

    g = @(x) exp(40.0*x)-1.0;

    A1 = diag(ones(N-1,1),-1) + diag([1.0;-ones(N-1,1)],0);
    M1 = [-1.0;ones(N-1,1)];
    A2 = diag([ones(N-1,1);0],0) - diag(ones(N-1,1),1);

    A3 = A1 - A2;
    A3(1,1) = -2.0;

    B = [1.0;zeros(N-1,1)];

    NON = @(x,u,p) M1.*g(A1*x) - g(A2*x) + (A3*p).*x + B*u;
    OUT = @(x,u,p) x(1);

    Q0 = 2.0*rand(P,100);
    QD = 2.0*rand(P,1);
    Q1 = ones(P,1)*[0,2.0];
    Q2 = ones(P,1);
    Q3 = Q2 + 0.1*randn(P,1);

    %% Cross-Gramian-Based Combined Reduction
    o = tic;
    Wj = emgr(NON,OUT,[J,N,O],T,'j',Q1,[1,0,0,0,0,0,0,0,1,1,0,0],U,0,0,1.0,0.02);
    [UX,DX,VX] = svd(Wj{1}); DX = diag(DX);
    [UI,DI,VI] = svd(Wj{2}); DI = diag(DI);
    OFFLINE_WJ = toc(o)

    o = tic;
    TIM01 = OFFLINE_WJ;
    ERR01 = cerror(@rk2,NON,OUT,T,X,U,Q0,UX,UX',UI,5);
    SAM01 = sample(@rk2,NON,OUT,T,X,U,QD,UX(:,1:20),UX(:,1:20)',UI(:,1:20));
    SAM03 = sample(@rk2,NON,OUT,T,X,U,QD,UX(:,1:50),UX(:,1:50)',UI(:,1:50));
    ONLINE_WJ = toc(o)
    save('../../data/ch5cmp2.mat','UX','UI','ERR01','TIM01','SAM01','SAM03','N','P','DX','DI');


    %% Optimization-Based Combined Reduction
    o = tic;
    XP = optmor(NON,OUT,[J,N,O],T,P,Q2,[0,0,0.1,0,0,0],U);
    if(size(XP{1},2)<N), XP{1}(N,N) = 0; end; UO = XP{1};
    if(size(XP{2},2)<P), XP{2}(P,P) = 0; end; UP = XP{2};
    OFFLINE_OP = toc(o)

    o = tic;
    TIM02 = TIMINGS;
    ERR02 = cerror(@rk2,NON,OUT,T,X,U,Q0,UO,UO',XP{2},5);
    SAM02 = sample(@rk2,NON,OUT,T,X,U,QD,UO(:,1:20),UO(:,1:20)',UP(:,1:20));
    SAM04 = sample(@rk2,NON,OUT,T,X,U,QD,UO(:,1:50),UO(:,1:50)',UP(:,1:50));
    SAM05 = sample(@rk2,NON,OUT,T,X,U,Q2,UO(:,1:50),UO(:,1:50)',UP(:,1:50));
    SAM06 = sample(@rk2,NON,OUT,T,X,U,Q3,UO(:,1:50),UO(:,1:50)',UP(:,1:50));
    ONLINE_OP = toc(o)
    save('../../data/ch5cmp2.mat','UO','UP','ERR02','TIM02','SAM02','SAM04','SAM05','SAM06','-append');
end
