function vernval_optmor()
% vernval_optmor (chapter 5 optmor: verification and validation)
% by Christian Himpe, 2015-2016 ( wwwmath.uni-muenster.de/u/himpe )
% released under BSD 2-Clause License ( opensource.org/licenses/BSD-2-Clause )
%*
    addpath('../');

    rand('seed',1009');
    randn('seed',1009');

%% SETUP

    J = 10;
    N = 100;
    O = J;
    T = [0.01,1.0];
    L = floor(T(2)/T(1)) + 1;

    X = zeros(N,1);
    U = [ones(J,1)./T(1),zeros(J,L)];
    P = N;

    [A,B,C] = ilp(J,N,O,1,1009);

    ISSTABLE = all(real(eig(A))<0)

    LIN = @(x,u,p) A*x + B*u + p;
    OUT = @(x,u,p) C*x;

    Q0 = rand(P,100);
    Q1 = 0.5*ones(P,1);
    qd = rand(P,1);
    YD = rk2(LIN,OUT,T,X,U,qd);
    QD = bsxfun(@plus,qd,0.1*randn(P,100));

    Y = rk2(LIN,OUT,[0.01,1.0],X,U,rand(P,1));

%% Varying Nominal
    for I=1:10
        o = tic;
        XP = optmor(LIN,OUT,[J,N,O],T,N,rand(P,1),[0,0,1e-1,0,0,0]);
        OFFLINE_T = toc(o)
        ERR01{I} = serror(@rk2,LIN,OUT,T,X,U,Q0,XP{1},XP{1}',4);
        ERR02{I} = perror(@rk2,LIN,OUT,T,X,U,Q0,XP{2},4);
        ERR03{I} = cerror(@rk2,LIN,OUT,T,X,U,Q0,XP{1},XP{1}',XP{2},4);
    end

    ERR01 = cat(3,ERR01{:});
    ERR02 = cat(3,ERR02{:});    
    ERR03 = cat(4,ERR03{:});

    ERR01 = cat(3,max(ERR01,[],3),mean(ERR01,3),min(ERR01,[],3));
    ERR02 = cat(3,max(ERR02,[],3),mean(ERR02,3),min(ERR02,[],3));
    ERR03 = cat(4,max(ERR03,[],4),mean(ERR03,4),min(ERR03,[],4));

    save('../../data/ch5optmor.mat','ERR01','ERR02','ERR03','N','P');


%% Tikhonov Regularization
    o = tic;
    XP = optmor(LIN,OUT,[J,N,O],T,P,Q1,[0,0,1e-16,0,0,0]);
    OFFLINE_T = toc(o)
    ERR11 = cerror(@rk2,LIN,OUT,T,X,U,Q0,XP{1},XP{1}',XP{2},10);
    save('../../data/ch5optmor.mat','ERR11','-append');

    o = tic;
    XP = optmor(LIN,OUT,[J,N,O],T,P,Q1,[0,0,1e-4,0,0,0]);
    OFFLINE_T = toc(o)
    ERR12 = cerror(@rk2,LIN,OUT,T,X,U,Q0,XP{1},XP{1}',XP{2},10);
    save('../../data/ch5optmor.mat','ERR12','-append');

    o = tic;
    XP = optmor(LIN,OUT,[J,N,O],T,P,Q1,[0,0,1e-3,0,0,0]);
    OFFLINE_T = toc(o)
    ERR13 = cerror(@rk2,LIN,OUT,T,X,U,Q0,XP{1},XP{1}',XP{2},10);
    save('../../data/ch5optmor.mat','ERR13','-append');

    o = tic;
    XP = optmor(LIN,OUT,[J,N,O],T,P,Q1,[0,0,1e-2,0,0,0]);
    OFFLINE_T = toc(o)
    ERR14 = cerror(@rk2,LIN,OUT,T,X,U,Q0,XP{1},XP{1}',XP{2},10);
    save('../../data/ch5optmor.mat','ERR14','-append');

    o = tic;
    XP = optmor(LIN,OUT,[J,N,O],T,P,Q1,[0,0,1e-1,0,0,0]);
    OFFLINE_T = toc(o)
    ERR15 = cerror(@rk2,LIN,OUT,T,X,U,Q0,XP{1},XP{1}',XP{2},10);
    save('../../data/ch5optmor.mat','ERR15','-append');

    o = tic;
    XP = optmor(LIN,OUT,[J,N,O],T,P,Q1,[0,0,1.0,0,0,0]);
    OFFLINE_T = toc(o)
    ERR16 = cerror(@rk2,LIN,OUT,T,X,U,Q0,XP{1},XP{1}',XP{2},10);
    save('../../data/ch5optmor.mat','ERR16','-append');


%% Number of Iterations
    o = tic;
    XP = optmor(LIN,OUT,[J,N,O],T,P,Q1,[0,0,1e-1,0,2,0]);
    OFFLINE_T = toc(o)
    ERR21 = cerror(@rk2,LIN,OUT,T,X,U,Q0,XP{1},XP{1}',XP{2},10);
    save('../../data/ch5optmor.mat','ERR21','-append');

    o = tic;
    XP = optmor(LIN,OUT,[J,N,O],T,P,Q1,[0,0,1e-1,0,3,0]);
    OFFLINE_T = toc(o)
    ERR22 = cerror(@rk2,LIN,OUT,T,X,U,Q0,XP{1},XP{1}',XP{2},10);
    save('../../data/ch5optmor.mat','ERR22','-append');

    o = tic;
    XP = optmor(LIN,OUT,[J,N,O],T,P,Q1,[0,0,1e-1,0,4,0]);
    OFFLINE_T = toc(o)
    ERR23 = cerror(@rk2,LIN,OUT,T,X,U,Q0,XP{1},XP{1}',XP{2},10);
    save('../../data/ch5optmor.mat','ERR23','-append');

    o = tic;
    XP = optmor(LIN,OUT,[J,N,O],T,P,Q1,[0,0,1e-1,0,5,0]);
    OFFLINE_T = toc(o)
    ERR24 = cerror(@rk2,LIN,OUT,T,X,U,Q0,XP{1},XP{1}',XP{2},10);
    save('../../data/ch5optmor.mat','ERR24','-append');

    o = tic;
    XP = optmor(LIN,OUT,[J,N,O],T,P,Q1,[0,0,1e-1,0,6,0]);
    OFFLINE_T = toc(o)
    ERR25 = cerror(@rk2,LIN,OUT,T,X,U,Q0,XP{1},XP{1}',XP{2},10);
    save('../../data/ch5optmor.mat','ERR25','-append');

    o = tic;
    XP = optmor(LIN,OUT,[J,N,O],T,P,Q1,[0,0,1e-1,0,10,0]);
    OFFLINE_T = toc(o)
    ERR26 = cerror(@rk2,LIN,OUT,T,X,U,Q0,XP{1},XP{1}',XP{2},10);
    save('../../data/ch5optmor.mat','ERR26','-append');


%% Error Threshold
    o = tic;
    XP = optmor(LIN,OUT,[J,N,O],T,1e-2,Q1,[0,0,1e-1,0,0,0]);
    if(size(XP{1},2)<N), XP{1}(N,N) = 0; end;
    if(size(XP{2},2)<P), XP{2}(P,P) = 0; end;
    OFFLINE_T = toc(o)
    ERR31 = cerror(@rk2,LIN,OUT,T,X,U,Q0,XP{1},XP{1}',XP{2},10);
    save('../../data/ch5optmor.mat','ERR31','-append');

    o = tic;
    XP = optmor(LIN,OUT,[J,N,O],T,1e-4,Q1,[0,0,1e-1,0,0,0]);
    if(size(XP{1},2)<N), XP{1}(N,N) = 0; end;
    if(size(XP{2},2)<P), XP{2}(P,P) = 0; end;
    OFFLINE_T = toc(o)
    ERR32 = cerror(@rk2,LIN,OUT,T,X,U,Q0,XP{1},XP{1}',XP{2},10);
    save('../../data/ch5optmor.mat','ERR32','-append');

    o = tic;
    XP = optmor(LIN,OUT,[J,N,O],T,1e-6,Q1,[0,0,1e-1,0,0,0]);
    if(size(XP{1},2)<N), XP{1}(N,N) = 0; end;
    if(size(XP{2},2)<P), XP{2}(P,P) = 0; end;
    OFFLINE_T = toc(o)
    ERR33 = cerror(@rk2,LIN,OUT,T,X,U,Q0,XP{1},XP{1}',XP{2},10);
    save('../../data/ch5optmor.mat','ERR33','-append');

    o = tic;
    XP = optmor(LIN,OUT,[J,N,O],T,1e-8,Q1,[0,0,1e-1,0,0,0]);
    if(size(XP{1},2)<N), XP{1}(N,N) = 0; end;
    if(size(XP{2},2)<P), XP{2}(P,P) = 0; end;
    OFFLINE_T = toc(o)
    ERR34 = cerror(@rk2,LIN,OUT,T,X,U,Q0,XP{1},XP{1}',XP{2},10);
    save('../../data/ch5optmor.mat','ERR34','-append');

    o = tic;
    XP = optmor(LIN,OUT,[J,N,O],T,1e-10,Q1,[0,0,1e-1,0,0,0]);
    if(size(XP{1},2)<N), XP{1}(N,N) = 0; end;
    if(size(XP{2},2)<P), XP{2}(P,P) = 0; end;
    OFFLINE_T = toc(o)
    ERR35 = cerror(@rk2,LIN,OUT,T,X,U,Q0,XP{1},XP{1}',XP{2},10);
    save('../../data/ch5optmor.mat','ERR35','-append');

    o = tic;
    XP = optmor(LIN,OUT,[J,N,O],T,1e-12,Q1,[0,0,1e-1,0,0,0]);
    if(size(XP{1},2)<N), XP{1}(N,N) = 0; end;
    if(size(XP{2},2)<P), XP{2}(P,P) = 0; end;
    OFFLINE_T = toc(o)
    ERR36 = cerror(@rk2,LIN,OUT,T,X,U,Q0,XP{1},XP{1}',XP{2},10);
    save('../../data/ch5optmor.mat','ERR36','-append');


%% Data-Driven Regularization
    o = tic;
    XP = optmor(LIN,OUT,[J,N,O],T,P,Q1,[0,0,1e-1,0,0,0]);
    OFFLINE_D1 = toc(o)
    ERR41 = cerror(@rk2,LIN,OUT,T,X,U,QD,XP{1},XP{1}',XP{2},10);
    save('../../data/ch5optmor.mat','ERR41','OFFLINE_D1','-append');

    o = tic;
    XP = optmor(LIN,OUT,[J,N,O],T,P,Q1,[0,0,1e-1,1e-12,0],1,0,1,YD);
    OFFLINE_D2 = toc(o)
    ERR42 = cerror(@rk2,LIN,OUT,T,X,U,QD,XP{1},XP{1}',XP{2},10);
    save('../../data/ch5optmor.mat','ERR42','OFFLINE_D2','-append');

    o = tic;
    XP = optmor(LIN,OUT,[J,N,O],T,P,Q1,[0,0,1e-1,1e-8,0],1,0,1,YD);
    OFFLINE_D3 = toc(o)
    ERR43 = cerror(@rk2,LIN,OUT,T,X,U,QD,XP{1},XP{1}',XP{2},10);
    save('../../data/ch5optmor.mat','ERR43','OFFLINE_D3','-append');

    o = tic;
    XP = optmor(LIN,OUT,[J,N,O],T,P,Q1,[0,0,1e-1,1e-4,0],1,0,1,YD);
    OFFLINE_D4 = toc(o)
    ERR44 = cerror(@rk2,LIN,OUT,T,X,U,QD,XP{1},XP{1}',XP{2},10);
    save('../../data/ch5optmor.mat','ERR44','OFFLINE_D4','-append');

    o = tic;
    XP = optmor(LIN,OUT,[J,N,O],T,P,Q1,[0,0,1e-1,1e-2,0],1,0,1,YD);
    OFFLINE_D5 = toc(o)
    ERR45 = cerror(@rk2,LIN,OUT,T,X,U,QD,XP{1},XP{1}',XP{2},10);
    save('../../data/ch5optmor.mat','ERR45','OFFLINE_D5','-append');

    o = tic;
    XP = optmor(LIN,OUT,[J,N,O],T,P,Q1,[0,0,1e-1,1e-1,0],1,0,1,YD);
    OFFLINE_D6 = toc(o)
    ERR46 = cerror(@rk2,LIN,OUT,T,X,U,QD,XP{1},XP{1}',XP{2},10);
    save('../../data/ch5optmor.mat','ERR46','OFFLINE_D6','-append');
end
