function ch7plot(o)
% ch7plot (plot results of chapter 7)
% by Christian Himpe, 2014-2015 ( wwwmath.uni-muenster.de/u/himpe )
% released under BSD 2-Clause License ( opensource.org/licenses/BSD-2-Clause )
%*
    addpath('../');

    if(nargin<1 || isempty(o)), o = 0; end;


    %% HNM
    load('../../data/ch7hnm.mat');

    % Surfaces
    plot_surf(ERR01(:,:,2),'ch7hnm_ng22',[N,P],1e-16,1);
    plot_surf(ERR02(:,:,2),'ch7hnm_no22',[N,P],1e-16,1);
    plot_surf(ERR01(:,:,4),'ch7hnm_ng28',[N,P],1e-16,1);
    plot_surf(ERR02(:,:,4),'ch7hnm_no28',[N,P],1e-16,1);

    plot_surf(ERR03(:,:,2),'ch7hnm_lg22',[N,P],1e-16,1);
    plot_surf(ERR04(:,:,2),'ch7hnm_lo22',[N,P],1e-16,1);
    plot_surf(ERR03(:,:,4),'ch7hnm_lg28',[N,P],1e-16,1);
    plot_surf(ERR04(:,:,4),'ch7hnm_lo28',[N,P],1e-16,1);

    % Slices
    plot_slice({ERR01(:,:,2),ERR02(:,:,2),ERR03(:,:,2),ERR04(:,:,2)},'ch7hnm_vs22',[N,P],1e-17);
    plot_slice({ERR01(:,:,4),ERR02(:,:,4),ERR03(:,:,4),ERR04(:,:,4)},'ch7hnm_vs28',[N,P],1e-17);

    % Offline
    plot_offline({TIM01,sum(reshape(TIM02,[10,P/10]),2),TIM03,sum(reshape(TIM04,[10,P/10]),2)},{'ch7hnm_off1','ch7hnm_off2'});
    OFFLINE_GRN = TIM01
    OFFLINE_GRL = TIM03
    OFFLINE_OPN = sum(TIM02)
    OFFLINE_OPL = sum(TIM04)
    AVG_OFFLINE_OPN = mean(TIM02)
    AVG_OFFLINE_OPL = mean(TIM04)


    %% MRI
    load('../../data/ch7mri.mat');

    ERR01(isnan(ERR01)) = 1.0;
    ERR02(isnan(ERR02)) = 1.0;
    ERR03(isnan(ERR03)) = 1.0;
    ERR04(isnan(ERR04)) = 1.0;

    % Surfaces
    plot_surf(ERR01(:,:,2),'ch7mri_ng22',[N,P],1e-16,1);
    plot_surf(ERR02(:,:,2),'ch7mri_no22',[N,P],1e-16,1);
    plot_surf(ERR01(:,:,4),'ch7mri_ng28',[N,P],1e-16,1);
    plot_surf(ERR02(:,:,4),'ch7mri_no28',[N,P],1e-16,1);

    plot_surf(ERR03(:,:,2),'ch7mri_lg22',[N,P],1e-16,1);
    plot_surf(ERR04(:,:,2),'ch7mri_lo22',[N,P],1e-16,1);
    plot_surf(ERR03(:,:,4),'ch7mri_lg28',[N,P],1e-16,1);
    plot_surf(ERR04(:,:,4),'ch7mri_lo28',[N,P],1e-16,1);

    % Slices
    plot_slice({ERR01(:,:,2),ERR02(:,:,2),ERR03(:,:,2),ERR04(:,:,2)},'ch7mri_vs22',[N,P],1e-17);
    plot_slice({ERR01(:,:,4),ERR02(:,:,4),ERR03(:,:,4),ERR04(:,:,4)},'ch7mri_vs28',[N,P],1e-17);

    % Offline
    plot_offline({TIM01,sum(reshape(TIM02,[16,P/16]),2),TIM03,sum(reshape(TIM04,[16,P/16]),2)},{'ch7mri_off1','ch7mri_off2'});
    OFFLINE_GRN = TIM01
    OFFLINE_GRL = TIM03
    OFFLINE_OPN = sum(TIM02)
    OFFLINE_OPL = sum(TIM04)
    AVG_OFFLINE_OPN = mean(TIM02)
    AVG_OFFLINE_OPL = mean(TIM04)


    %% EEG
    load('../../data/ch7eeg.mat');

    % Surfaces
    plot_surf(ERR01(:,:,2),'ch7eeg_ng22',[N,P],1e-16,1);
    plot_surf(ERR02(:,:,2),'ch7eeg_no22',[N,P],1e-16,1);
    plot_surf(ERR01(:,:,4),'ch7eeg_ng28',[N,P],1e-16,1);
    plot_surf(ERR02(:,:,4),'ch7eeg_no28',[N,P],1e-16,1);

    plot_surf(ERR03(:,:,2),'ch7eeg_lg22',[N,P],1e-16,1);
    plot_surf(ERR04(:,:,2),'ch7eeg_lo22',[N,P],1e-16,1);
    plot_surf(ERR03(:,:,4),'ch7eeg_lg28',[N,P],1e-16,1);
    plot_surf(ERR04(:,:,4),'ch7eeg_lo28',[N,P],1e-16,1);

    % Slices
    plot_slice({ERR01(:,:,2),ERR02(:,:,2),ERR03(:,:,2),ERR04(:,:,2)},'ch7eeg_vs22',[N,P],1e-17);
    plot_slice({ERR01(:,:,4),ERR02(:,:,4),ERR03(:,:,4),ERR04(:,:,4)},'ch7eeg_vs28',[N,P],1e-17);

    % Offline
    plot_offline({TIM01,sum(reshape(TIM02,[16,P/16]),2),TIM03,sum(reshape(TIM04,[16,P/16]),2)},{'ch7eeg_off1','ch7eeg_off2'});
    OFFLINE_GRN = TIM01
    OFFLINE_GRL = TIM03
    OFFLINE_OPN = sum(TIM02)
    OFFLINE_OPL = sum(TIM04)
    AVG_OFFLINE_OPN = mean(TIM02)
    AVG_OFFLINE_OPL = mean(TIM04)


    % Bayesian Inversion
    load('../../data/ch7bayinv.mat');

    % Data
    figure;
    plot(SAM01(1,:),YD,'Color',colors('blue'),'Linewidth',8);    
    pbaspect([1.5,1.6,1]);
    set([gca; findall(gca,'Type','text')],'FontSize',16);
    print('-dpdf',['../../plots/ch7bim.pdf']);

    % fMRI results
    figure;
    plot(SAM01(1,:),YD,'Color',colors('blue'),'Linewidth',8);
    hold on;
    plot(SAM01(1,:),SAM01(2:end,:),'Color',colors('red'),'Linewidth',8);
    hold off;
    pbaspect([1.5,1.6,1]);
    set([gca; findall(gca,'Type','text')],'FontSize',16);
    print('-dpdf',['../../plots/ch7bimf.pdf']);

    figure;
    plot(SAM02(1,:),YD,'Color',colors('blue'),'Linewidth',8);
    hold on;
    plot(SAM02(1,:),SAM02(2:end,:),'Color',colors('red'),'Linewidth',8);
    hold off;
    pbaspect([1.5,1.6,1]);
    set([gca; findall(gca,'Type','text')],'FontSize',16);
    print('-dpdf',['../../plots/ch7bimg.pdf']);

    figure;
    plot(SAM03(1,:),YD,'Color',colors('blue'),'Linewidth',8);
    hold on;
    plot(SAM03(1,:),SAM03(2:end,:),'Color',colors('red'),'Linewidth',8);
    hold off;
    pbaspect([1.5,1.6,1]);
    set([gca; findall(gca,'Type','text')],'FontSize',16);
    print('-dpdf',['../../plots/ch7bimo.pdf']);

    figure;
    plot(SAM04(1,:),YD,'Color',colors('blue'),'Linewidth',8);
    hold on;
    plot(SAM04(1,:),SAM04(2:end,:),'Color',colors('red'),'Linewidth',8);
    hold off;
    pbaspect([1.5,1.6,1]);
    set([gca; findall(gca,'Type','text')],'FontSize',16);
    print('-dpdf',['../../plots/ch7bimd.pdf']);
end
