function eeg()
% eeg (Chapter 7 EEG & MEG Dynamic Causal Model Experiments)
% by Christian Himpe, 2015-2016 ( wwwmath.uni-muenster.de/u/himpe )
% released under BSD 2-Clause License ( opensource.org/licenses/BSD-2-Clause )
%*
    global TIMINGS;

    addpath('../');

    rand('seed',1009);
    randn('seed',1009);

%% SETUP

    n = 16;
    J = 1;
    N = 10*n;
    O = 16;
    P = n*n;
    T = [1.0,200.0];
    L = floor(T(2)/T(1)) + 1;

    X = zeros(N,1);
    U = [ones(J,1)./T(1),zeros(J,L)];

    a = full(sprand(n,n,1.0/n))*32.0;
    a(1:n+1:end) = 0;
    b = full(sprand(n,J,1.0/n));

    v = ones(P,1); v(1:n+1:end) = 0.01;

    [NON,LIN,OUT] = eeg_util(n,a,b);

    Q0 = exp(0.25*bsxfun(@times,randn(n*n,100),v));
    QD = exp(0.25*bsxfun(@times,randn(n*n,1),v));
    Q1 = [0.5*v.*ones(P,1),2.0*v.*ones(P,1)];
    Q2 = ones(P,1);

%% MAIN NONLINEAR

    o = tic;
    WJ = emgr(NON,OUT,[J,N,O],T,'j',Q1,[1,0,0,3,0,0,1,0,1,1,0,0]);
    [UXN,DXN,vxn] = svd(WJ{1}(1:5*n,1:5*n));
    %[UXN,DXN,vxn] = svd(WJ{1}(5*n+1:end,5*n+1:end));
    [UIN,DIN,vin] = svd(WJ{2});
    OFFLINE_WJN = toc(o)

    o = tic;
    TIM01 = OFFLINE_WJN;
    ERR01 = cerror_so(@rk2,NON,OUT,T,X,U,Q0,UXN,UXN',UIN,4);
    ERR01(ERR01>1.0) = 1.0;
    uxn = [UXN(:,1:40),zeros(5*n,40);zeros(5*n,40),UXN(:,1:40)];
    SAM01 = sample(@rk2,NON,OUT,T,X,U,QD,uxn,uxn',UIN(:,1:40));
    ONLINE_WJN = toc(o)
    save('../../data/ch7eeg2.mat','UXN','UIN','ERR01','SAM01','TIM01','N','P');

    PROGRESS = 0.25

    o = tic;
    XPN = optmor(NON,OUT,[J,N,O],T,P,Q2,[0,0,0.1,0,0,0]);
    UON = XPN{1}(1:5*n,1:5*n);
    UPN = XPN{2};
    OFFLINE_OPN = toc(o)

    o = tic;
    TIM02 = TIMINGS;
    ERR02 = cerror_so(@rk2,NON,OUT,T,X,U,Q0,UON,UON',UPN,4);
    ERR02(ERR02>1.0) = 1.0;
    uon = [UON(:,1:40),zeros(5*n,40);zeros(5*n,40),UON(:,1:40)];
    SAM02 = sample(@rk2,NON,OUT,T,X,U,QD,uon,uon',UPN(:,1:40));
    ONLINE_OPN = toc(o)
    save('../../data/ch7eeg2.mat','UON','UPN','ERR02','SAM02','TIM02','-append');

    PROGRESS = 0.5

%% MAIN LINEARIZED

    o = tic;
    WJ = emgr(LIN,OUT,[J,N,O],T,'j',Q1,[1,0,0,3,0,0,1,0,1,1,0,0]);
    [UXL,DXL,vxl] = svd(WJ{1}(1:5*n,1:5*n));
    %[UXL,DXL,vxl] = svd(WJ{1}(5*n+1:end,5*n+1:end));
    [UIL,DIL,vil] = svd(WJ{2},N);
    OFFLINE_WJL = toc(o)

    o = tic;
    TIM03 = OFFLINE_WJL;
    ERR03 = cerror_so(@rk2,NON,OUT,T,X,U,Q0,UXL,UXL',UIL,4);
    ERR03(ERR03>1.0) = 1.0;
    uxl = [UXL(:,1:40),zeros(5*n,40);zeros(5*n,40),UXL(:,1:40)];
    SAM03 = sample(@rk2,NON,OUT,T,X,U,QD,uxl,uxl',UIL(:,1:40));
    ONLINE_WJL = toc(o)
    save('../../data/ch7eeg2.mat','UXL','UIL','ERR03','SAM03','TIM03','-append');

    PROGESS = 0.75

    o = tic;
    XPL = optmor(LIN,OUT,[J,N,O],T,P,Q2,[0,0,0.1,0,0,0]);
    UOL = XPL{1}(1:5*n,1:5*n);
    UPL = XPL{2};
    OFFLINE_OPL = toc(o) 

    o = tic;
    TIM04 = TIMINGS;
    ERR04 = cerror_so(@rk2,NON,OUT,T,X,U,Q0,UOL,UOL',UPL,4);
    ERR04(ERR04>1.0) = 1.0;
    uol = [UOL(:,1:40),zeros(5*n,40);zeros(5*n,40),UOL(:,1:40)];
    SAM04 = sample(@rk2,NON,OUT,T,X,U,QD,uol,uol',UPL(:,1:40));
    ONLINE_OPL = toc(o)
    save('../../data/ch7eeg2.mat','UOL','UPL','ERR04','SAM04','TIM04','-append');

    PROGRESS = 1.0
end
