function hnm()
% hnm (Chapter 7 Hyperbolic Network Model Experiments)
% by Christian Himpe, 2015-2016 ( wwwmath.uni-muenster.de/u/himpe )
% released under BSD 2-Clause License ( opensource.org/licenses/BSD-2-Clause )
%*
    global TIMINGS;

    addpath('../');

    rand('seed',1009);
    randn('seed',1009);

%% SETUP

    J = 1;
    N = 100;
    O = J;
    P = N;
    T = [0.01,2.0];
    L = floor(T(2)/T(1)) + 1;

    X = zeros(N,1);
    U = [ones(J,1)./T(1),zeros(J,L)];

    A = full(sprand(N,N,1./N));
    A(1:N+1:end) = -1.0;
    A = A * 10.0;

    IS_STABLE = all(real(eig(full(A)))<0)

    B = 10.0*full(sprand(N,J,10.0/N));
    C = rand(O,N);

    LIN = @(x,u,p) A*(p.*x) + B*u;
    NON = @(x,u,p) A*tanh(p.*x) + B*u;
    OUT = @(x,u,p) C*x;

    Q0 = 0.9*rand(N,100)+0.1;
    QD = 0.9*rand(N,1)+0.1;
    Q1 = [0.1*ones(N,1),ones(N,1)];
    Q2 = 0.55*ones(N,1);

%% MAIN NONLINEAR

    o = tic;
    WJ = emgr(NON,OUT,[J,N,O],T,'j',Q1,[0,0,0,0,0,0,0,0,1,1,0,0]);
    [UXN,DXN,vxn] = svd(WJ{1});
    [UIN,DIN,vin] = svd(WJ{2});
    OFFLINE_WJN = toc(o)

    o = tic;
    TIM01 = OFFLINE_WJN;
    ERR01 = cerror(@rk2,NON,OUT,T,X,U,Q0,UXN,UXN',UIN,5);
    ERR01(ERR01>1.0) = 1.0;
    SAM01 = sample(@rk2,NON,OUT,T,X,U,QD,UXN(:,1:60),UXN(:,1:60)',UIN(:,1:60));
    ONLINE_WJN = toc(o)
    save('../../data/ch7hnm2.mat','UXN','UIN','ERR01','SAM01','TIM01','N','P');

    PROGRESS = 0.25

    o = tic;
    XPN = optmor(NON,OUT,[J,N,O],T,P,Q2,[0,0,0.01,0,0,0]);
    UON = XPN{1};
    UPN = XPN{2};
    OFFLINE_OPN = toc(o)

    o = tic;
    TIM02 = TIMINGS;
    ERR02 = cerror(@rk2,NON,OUT,T,X,U,Q0,UON,UON',UPN,5);
    ERR02(ERR02>1.0) = 1.0;
    SAM02 = sample(@rk2,NON,OUT,T,X,U,QD,UON(:,1:60),UON(:,1:60)',UPN(:,1:60));
    ONLINE_OPN = toc(o)
    save('../../data/ch7hnm2.mat','UON','UPN','ERR02','SAM02','TIM02','-append');

    PROGRESS = 0.50

%% MAIN LINEARIZED

    o = tic;
    WJ = emgr(LIN,OUT,[J,N,O],T,'j',Q1,[0,0,0,0,0,0,0,0,1,1,0,0]);
    [UXL,DXL,vxl] = svd(WJ{1});
    [UIL,DIL,vil] = svd(WJ{2});
    OFFLINE_WJL = toc(o)

    o = tic;
    TIM03 = OFFLINE_WJL;
    ERR03 = cerror(@rk2,NON,OUT,T,X,U,Q0,UXL,UXL',UIL,5);
    ERR03(ERR03>1.0) = 1.0;
    SAM03 = sample(@rk2,NON,OUT,T,X,U,QD,UXL(:,1:60),UXL(:,1:60)',UIL(:,1:60));
    ONLINE_WJL = toc(o)
    save('../../data/ch7hnm2.mat','UXL','UIL','ERR03','SAM03','TIM03','-append');

    PROGRESS = 0.75

    o = tic;
    XPL = optmor(LIN,OUT,[J,N,O],T,P,Q2,[0,0,0.01,0,0,0]);
    UOL = XPL{1};
    UPL = XPL{2};
    OFFLINE_OPL = toc(o)

    o = tic;
    TIM04 = TIMINGS;
    ERR04 = cerror(@rk2,NON,OUT,T,X,U,Q0,UOL,UOL',UPL,5);
    ERR04(ERR04>1.0) = 1.0;
    SAM04 = sample(@rk2,NON,OUT,T,X,U,QD,UOL(:,1:60),UOL(:,1:60)',UPL(:,1:60));
    ONLINE_OPL = toc(o)
    save('../../data/ch7hnm2.mat','UOL','UPL','ERR04','SAM04','TIM04','-append');

    PROGRESS = 1.00
end
