function mri()
% mri (Chapter 7 fMRI & fNIRS Dynamic Causal Model Experiments)
% by Christian Himpe, 2015-2016 ( wwwmath.uni-muenster.de/u/himpe )
% released under BSD 2-Clause License ( opensource.org/licenses/BSD-2-Clause )
%*
    global TIMINGS;

    addpath('../');

    rand('seed',1009);
    randn('seed',1009);

%% SETUP

    n = 16;
    J = 1;
    N = 5*n;
    O = n;
    P = n*n;
    T = [0.1,20.0];
    L = floor(T(2)/T(1)) + 1;

    XL = zeros(N,1);
    XN = [zeros(2*n,1);ones(3*n,1)];
    U = [ones(J,1)./T(1),zeros(J,L)];

    a = full(sprand(n,n,1.0/n));
    a(1:n+1:end) = -2.0;
    b = full(sprand(n,J,1.0/n));

    v1 = 0.5*ones(P,1); v(1:n+1:end) = 0.95;
    v2 = 1.5*ones(P,1); v(1:n+1:end) = 1.05;

    [NON,NOU,LIN,OUT] = mri_util(n,a,b);

    v = ones(P,1); v(1:n+1:end) = 0.01;

    Q0 = exp(0.25*bsxfun(@times,randn(n*n,100),v));
    QD = exp(0.25*bsxfun(@times,randn(n*n,1),v));
    Q1 = [v1,v2];
    Q2 = ones(P,1);

%% MAIN NONLINEAR

    o = tic;
    WJ = emgr(NON,NOU,[J,N,O],T,'j',Q1,[2,0,0,3,0,0,1,0,1,1,0,0],1.0,0,XN,1.0,0.9);
    [UXN,DXN,vxn] = svd(WJ{1}+WJ{1}');
    [UIN,DIN,vin] = svd(WJ{2});
    OFFLINE_WJN = toc(o)

    o = tic;
    TIM01 = OFFLINE_WJN;
    ERR01 = cerror(@rk2,NON,NOU,T,XN,U,Q0,vxn,vxn',UIN,4);
    ERR01(ERR01>1.0) = 1.0;
    SAM01 = sample(@rk2,NON,NOU,T,XN,U,QD,vxn(:,1:44),vxn(:,1:44)',UIN(:,1:44));
    ONLINE_WJN = toc(o)
    save('../../data/ch7mri2.mat','vxn','UIN','ERR01','SAM01','TIM01','N','P');

    PROGRESS = 0.25

    o = tic;
    XPN = optmor(NON,NOU,[J,N,O],T,n*n,Q2,[0,0,0.1,0,0,0],1,XN);
    UON = XPN{1};
    UPN = XPN{2};
    OFFLINE_OPN = toc(o)

    o = tic;
    TIM02 = TIMINGS;
    ERR02 = cerror(@rk2,NON,NOU,T,XN,U,Q0,UON,UON',UPN,4);
    ERR02(ERR02>1.0) = 1.0;
    SAM02 = sample(@rk2,NON,NOU,T,XN,U,QD,UON(:,1:24),UON(:,1:24)',UPN(:,1:24));
    ONLINE_OPN = toc(o)
    save('../../data/ch7mri2.mat','UON','UPN','ERR02','SAM02','TIM02','-append');

    PROGRESS = 0.5

%% MAIN LINEARIZED

    o = tic;
    WJ = emgr(LIN,OUT,[J,N,O],T,'j',Q1,[2,0,0,3,0,0,1,0,1,1,0,0],1.0,0,XL,1.0,0.9);
    [UXL,DXL,vxl] = svd(WJ{1}+WJ{1}');
    [UIL,DIL,vil] = svd(WJ{2});
    OFFLINE_WJL = toc(o)

    o = tic;
    TIM03 = OFFLINE_WJL;
    ERR03 = cerror(@rk2,NON,NOU,T,XN,U,Q0,vxl,vxl',UIL,4);
    ERR03(ERR03>1.0) = 1.0;
    SAM03 = sample(@rk2,NON,NOU,T,XN,U,QD,vxl(:,1:44),vxl(:,1:44)',UIL(:,1:44));
    ONLINE_WJL = toc(o)
    save('../../data/ch7mri2.mat','vxl','UIL','ERR03','SAM03','TIM03','-append');

    PROGRESS = 0.75

    o = tic;
    XPL = optmor(LIN,OUT,[J,N,O],T,n*n,Q2,[0,0,0.1,0,0,0],1,XL);
    UOL = XPL{1};
    UPL = XPL{2};
    OFFLINE_OPL = toc(o)

    o = tic;
    TIM04 = TIMINGS;
    ERR04 = cerror(@rk2,NON,NOU,T,XN,U,Q0,UOL,UOL',UPL,4);
    ERR04(ERR04>1.0) = 1.0;
    SAM04 = sample(@rk2,NON,NOU,T,XN,U,QD,UOL(:,1:24),UOL(:,1:24)',UPL(:,1:24));
    ONLINE_OPL = toc(o)
    save('../../data/ch7mri2.mat','UOL','UPL','ERR04','SAM04','TIM04','-append');

    PROGRESS = 1.00
end
