function ch8plot(o)
% ch8plot (plot results of chapter 8)
% by Christian Himpe, 2016 ( wwwmath.uni-muenster.de/u/himpe )
% released under BSD 2-Clause License ( opensource.org/licenses/BSD-2-Clause )
%*
    addpath('../');

    if(nargin<1 || isempty(o)), o = 0; end;


    load('../../data/ch7hnm.mat');

    n = size(ERR01,1);

    tim02_hnm = cumsum(sum(reshape(TIM02,[5,n+1]),1));
    tim04_hnm = cumsum(sum(reshape(TIM04,[5,n+1]),1));

    time_hnm = [TIM01*ones(1,n), TIM03*ones(1,n); ...
                tim02_hnm(1:n),  tim04_hnm(1:n)];

    error_hnm = [diag(ERR01(:,:,2))', diag(ERR03(:,:,2))'; ...
                 diag(ERR02(:,:,2))', diag(ERR04(:,:,2))'];


    load('../../data/ch7mri.mat');

    n = size(ERR01,1);

    tim02_mri = cumsum(sum(reshape(TIM02(1:N),[4,n+1]),1));
    tim04_mri = cumsum(sum(reshape(TIM04(1:N),[4,n+1]),1));

    time_mri = [TIM01*ones(1,n), TIM03*ones(1,n); ...
                tim02_mri(1:n), tim04_mri(1:n)];

    error_mri = [diag(ERR01(:,:,2))', diag(ERR03(:,:,2))'; ...
                 diag(ERR02(:,:,2))', diag(ERR04(:,:,2))'];


    load('../../data/ch7eeg.mat');

    n = size(ERR01,1);

    tim02_eeg = cumsum(sum(reshape(TIM02(1:N/2),[4,n+1]),1));
    tim04_eeg = cumsum(sum(reshape(TIM04(1:N/2),[4,n+1]),1));

    time_eeg = [TIM01*ones(1,n), TIM03*ones(1,n); ...
                tim02_eeg(1:n),  tim04_eeg(1:n)];

    error_eeg = [diag(ERR01(:,:,2))', diag(ERR03(:,:,2))'; ...
                 diag(ERR02(1:19,:,2))', diag(ERR04(1:19,:,2))']; % TEMP


    figure; % Time vs Error
    loglog([time_hnm(1,:),time_mri(1,:),time_eeg(1,:)],[error_hnm(1,:),error_mri(1,:),error_eeg(1,:)],'Color',colors('red'),'linestyle','none','marker','*','linewidth',8); hold on;
    loglog([time_hnm(2,:),time_mri(2,:),time_eeg(2,:)],[error_hnm(2,:),error_mri(2,:),error_eeg(2,:)],'Color',colors('blue'),'linestyle','none','marker','*','linewidth',8); hold off;
    ylim([1e-16,1.5]);
    %annotation('textarrow',[0.9,0.8],[0.9,0.8]);
    l = legend('Gramian-Based','Optimization-Based','location','southwest');
    set(l,'FontSize',16);
    set([gca; findall(gca,'Type','text')],'FontSize',16);
    xlabel('Offline Time');
    ylabel('Relative Error');

    print('-dpdf',['../../plots/ch8eff.pdf']);
end
