function BND = sbound(ODE,DYN,OUT,T,X,U,P,UX,DX,VX,n,B,C)
% sbound (state-space bound computation)
% by Christian Himpe, 2014-2016 ( wwwmath.uni-muenster.de/u/himpe )
% released under BSD 2-Clause License ( opensource.org/licenses/BSD-2-Clause )
%*
    N = numel(X);
    h = T(1);

    BND = zeros(3,floor(N/n));

    Y = ODE(DYN,OUT,T,X,U,P);

    n1 = h*norm(Y(:),1);
    n2 = sqrt(h)*norm(Y(:),2);
    n8 = norm(Y(:),Inf);

    j = 1;

    for J=[1,n:n:(N-n)]

        BND(1,j) = (4.0*(N+J)*sum(DX(J+1:end)))/n1;							% approx.: 4.0*(N+J)*(N-J-1)*DX(J+1)
        BND(2,j) = (2.0*sum(DX(J+1:end)))/n2;								% approx.: 2.0*(N-J)*DX(J+1)
        BND(3,j) = sqrt(abs(trace( (C*UX(:,J+1:end))*diag(DX(J+1:end))*(VX(J+1:end,:)*B) )))/n8;	% approx.: sqrt(DX(J+1)*trace( (C*UX(:,J+1:end))*(VX(J+1:end,:)*B) ))

        j = j + 1;
    end
end 
