function ERR = serror(ODE,DYN,OUT,T,X,U,P,UX,VX,n)
% serror (state-space error evaluation)
% by Christian Himpe, 2014-2016 ( wwwmath.uni-muenster.de/u/himpe )
% released under BSD 2-Clause License ( opensource.org/licenses/BSD-2-Clause )
%*
    N = size(UX,2);
    Q = size(P,2);

    TMP = zeros(3,floor(N/n),Q);

    for I=1:Q

        Y = ODE(DYN,OUT,T,X,U,P(:,I));

        n1 = norm(Y(:),1);
        n2 = norm(Y(:),2);
        n8 = norm(Y(:),Inf);

        j = 1;

        for J=[1,n:n:(N-n)]

            ux = UX(:,1:J);
            vx = VX(1:J,:);

            if(any(X))
                dyn = @(x,u,p) vx*DYN(X + ux*x,u,p);
                out = @(x,u,p) OUT(X + ux*x,u,p);
            else
                dyn = @(x,u,p) vx*DYN(ux*x,u,p);
                out = @(x,u,p) OUT(ux*x,u,p);
            end

            x = zeros(J,1);

            y = ODE(dyn,out,T,x,U,P(:,I));

            TMP(1,j,I) = norm(Y(:)-y(:),1)/n1;
            TMP(2,j,I) = norm(Y(:)-y(:),2)/n2;
            TMP(3,j,I) = norm(Y(:)-y(:),Inf)/n8;

            j = j + 1;
        end
    end

    ERR = [sqrt(sum(TMP.^2,3));max(TMP(2,:,:),[],3)];
end
